/** Bookshelf - BorrowBookAction.java - 16/03/2007 */
package net.java.dev.esjug.bookshelf.controller;

import java.util.Collection;

import net.java.dev.esjug.bookshelf.application.BorrowBookService;
import net.java.dev.esjug.bookshelf.application.RegisterBookService;
import net.java.dev.esjug.bookshelf.application.RegisterMemberService;
import net.java.dev.esjug.bookshelf.domain.Book;
import net.java.dev.esjug.bookshelf.domain.Loan;
import net.java.dev.esjug.bookshelf.domain.Member;

import com.opensymphony.xwork2.ActionSupport;

/**
 * TODO: documentation.
 *
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public class BorrowBookAction extends ActionSupport {
	/** Serialization id. */
	private static final long serialVersionUID = 5554534874432130556L;
	
	/** Service class for the Register Member use case. */
	private RegisterMemberService registerMemberService;
	
	/** Service class for the Register Book use case. */
	private RegisterBookService registerBookService;
	
	/** Service class for the Borrow Book use case. */
	private BorrowBookService borrowBookService;

	/** The book that is being borrowed/returned. */
	private Book book = new Book();
	
	/** Id of the member that is borrowing the book. */
	private Long borrowerId;
	
	/** The book loan being created by a borrowing or closed by a book return. */
	private Loan loan = new Loan();
	
	/** Members that can borrow a book. */
	private Collection<Member> members;

	/**
	 * Setter for registerMemberService.
	 * @param registerMemberService The registerMemberService to set.
	 */
	public void setRegisterMemberService(RegisterMemberService registerMemberService) {
		this.registerMemberService = registerMemberService;
	}

	/**
	 * Setter for registerBookService.
	 * @param registerBookService The registerBookService to set.
	 */
	public void setRegisterBookService(RegisterBookService registerBookService) {
		this.registerBookService = registerBookService;
	}

	/**
	 * Setter for borrowBookService.
	 * @param borrowBookService The borrowBookService to set.
	 */
	public void setBorrowBookService(BorrowBookService borrowBookService) {
		this.borrowBookService = borrowBookService;
	}

	/**
	 * Getter for book.
	 * @return The book
	 */
	public Book getBook() {
		return book;
	}

	/**
	 * Getter for loan.
	 * @return The loan
	 */
	public Loan getLoan() {
		return loan;
	}

	/**
	 * Getter for members.
	 * @return The members
	 */
	public Collection<Member> getMembers() {
		if (members == null) members = registerMemberService.list();
		return members;
	}

	/**
	 * Setter for borrowerId.
	 * @param borrowerId The borrowerId to set.
	 */
	public void setBorrowerId(Long borrowerId) {
		this.borrowerId = borrowerId;
	}

	/* @see com.opensymphony.xwork2.ActionSupport#input() */
	@Override
	public String input() throws Exception {
		book = registerBookService.retrieve(book.getId());
		return super.input();
	}
	
	/**
	 * Action method that executes the use case "Borrow Book".
	 * 
	 * @return Result code for the action.
	 * @throws Exception If anything at all goes wrong.
	 */
	public String executeBorrow() throws Exception {
		Member borrower = registerMemberService.retrieve(borrowerId);
		book = registerBookService.retrieve(book.getId());
		loan = borrowBookService.borrowBook(borrower, book);
		return SUCCESS;
	}
	
	/**
	 * Action method that executes the use case "Return Book".
	 * 
	 * @return Result code for the action.
	 * @throws Exception If anything at all goes wrong.
	 */
	public String executeReturn() throws Exception {
		loan = borrowBookService.viewLoan(loan.getId());
		borrowBookService.returnBook(loan);
		return SUCCESS;
	}
}
