/** Bookshelf - SearchBooksAction.java - 15/03/2007 */
package net.java.dev.esjug.bookshelf.controller;

import java.util.Collection;

import net.java.dev.esjug.bookshelf.application.SearchBooksService;
import net.java.dev.esjug.bookshelf.domain.Book;

import com.opensymphony.xwork2.ActionSupport;

/**
 * TODO: documentation.
 *
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public class SearchBooksAction extends ActionSupport {
	/** Serialization id. */
	private static final long serialVersionUID = 1224609694106612833L;
	
	/** Represents the type of search by book name. */
	private static final int SEARCH_TYPE_BY_NAME = 1;
	
	/** Represents the type of search by authors name. */
	private static final int SEARCH_TYPE_BY_AUTHORS = 2;
	
	/** Represents the listing of available books. */
	private static final int SEARCH_TYPE_AVAILABLE_BOOKS = 3;
	
	/** Service class for the 'Search Books' use case. */
	private SearchBooksService searchBooksService;
	
	/** Search type. */
	private int searchType = SEARCH_TYPE_BY_NAME;
	
	/** Search key: the name (of the book or the author). */
	private String name;
	
	/** Search result. */
	private Collection<Book> books;
	
	/**
	 * Getter for name.
	 * @return The name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setter for name.
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Getter for searchType.
	 * @return The searchType
	 */
	public int getSearchType() {
		return searchType;
	}

	/**
	 * Setter for searchType.
	 * @param searchType The searchType to set.
	 */
	public void setSearchType(int searchType) {
		this.searchType = searchType;
	}

	/**
	 * Setter for searchBooksService.
	 * @param searchBooksService The searchBooksService to set.
	 */
	public void setSearchBooksService(SearchBooksService searchBooksService) {
		this.searchBooksService = searchBooksService;
	}

	/**
	 * Getter for books.
	 * @return The books
	 */
	public Collection<Book> getBooks() {
		return books;
	}

	/* @see com.opensymphony.xwork2.ActionSupport#execute() */
	@Override
	public String execute() throws Exception {
		// Determines the type of the search and performs it.
		switch (searchType) {
		case SEARCH_TYPE_BY_NAME:
			books = searchBooksService.searchByName(name);
			break;
		case SEARCH_TYPE_BY_AUTHORS:
			books = searchBooksService.searchByAuthorName(name);
			break;
		}
		return SUCCESS;
	}
	
	/**
	 * Action method that displays the list of available books.
	 * 
	 * @return Result code for the action.
	 * @throws Exception If anything at all goes wrong.
	 */
	public String executeListAvailableBooks() throws Exception {
		searchType = SEARCH_TYPE_AVAILABLE_BOOKS;
		books = searchBooksService.listAvailableBooks();
		return SUCCESS;
	}
}
