/** Bookshelf - Book.java - 15/03/2007 */
package net.java.dev.esjug.bookshelf.domain;

import java.util.Date;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import net.java.dev.esjug.util.persistence.PersistentObjectSupport;

import org.hibernate.annotations.Sort;
import org.hibernate.annotations.SortType;

/**
 * TODO: documentation.
 *
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
@Entity
@NamedQueries ({
	@NamedQuery(name="book.searchByName", query="from Book as book where book.name LIKE :name"),
	@NamedQuery(name="book.searchByAuthorName", query="from Book as book where book.authors LIKE :name"),
	@NamedQuery(name="book.retrieveAvailableBooks", query="from Book as book where book.id not in (select loan.book.id from Loan as loan where loan.returnDate is null)")
})
public class Book extends PersistentObjectSupport<Long, Long> implements Comparable<Book> {
	/** Serialization id. */
	private static final long serialVersionUID = 6355768580336868803L;

	/** Title of the book. */
	@Basic @Column(length = 200)
	private String name;
	
	/** Names of the authors of the book. */
	@Basic @Column(length = 250)
	private String authors;
	
	/** Register date. */
	@Temporal(TemporalType.DATE)
	private Date registerDate = new Date(System.currentTimeMillis());
	
	/** Book loans. */
	@OneToMany(mappedBy = "book") @Sort(type = SortType.NATURAL)
	private SortedSet<Loan> loans = new TreeSet<Loan>();
	
	/** Empty onstructor. */
	public Book() { }

	/**
	 * TODO: documentation.
	 * @param name Book title.
	 * @param authors Book authors.
	 */
	public Book(String name, String authors) {
		this.name = name;
		this.authors = authors;
	}

	/**
	 * Getter for authors.
	 * @return The authors
	 */
	public String getAuthors() {
		return authors;
	}

	/**
	 * Setter for authors.
	 * @param authors The authors to set.
	 */
	public void setAuthors(String authors) {
		this.authors = authors;
	}

	/**
	 * Getter for name.
	 * @return The name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setter for name.
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Getter for registerDate.
	 * @return The registerDate
	 */
	public Date getRegisterDate() {
		return registerDate;
	}

	/**
	 * Setter for registerDate.
	 * @param registerDate The registerDate to set.
	 */
	@SuppressWarnings("unused")
	private void setRegisterDate(Date registerDate) {
		this.registerDate = registerDate;
	}
	
	/**
	 * Getter for loans.
	 * @return The loans
	 */
	@SuppressWarnings("unused")
	private SortedSet<Loan> getLoans() {
		return loans;
	}

	/**
	 * Setter for loans.
	 * @param loans The loans to set.
	 */
	@SuppressWarnings("unused")
	private void setLoans(SortedSet<Loan> loans) {
		this.loans = loans;
	}
	
	/**
	 * Adds a loan to the end of the list of loans. Meant to be used by the class Loan only.
	 * 
	 * @param loan The loan to be added, to which this book has just been associated.
	 */
	protected void addLoan(Loan loan) {
		loans.add(loan);
	}

	/**
	 * Obtains the current loan of the book, if any.
	 * 
	 * @return The only open loan for this book or <code>null</code> if there isn't any.
	 */
	public Loan getCurrentLoan() {
		if ((loans == null) || (loans.size() == 0)) return null;
		Loan lastLoan = loans.last();
		return lastLoan.isClosed() ? null : lastLoan;
	}

	/**
	 * Indicates if the book is currently borrowed.
	 * 
	 * @return <code>true</code> if the book is currently borrowed, <code>false</code> otherwise.
	 */
	public boolean isBorrowed() {
		return getCurrentLoan() != null;
	}

	/**
	 * Copies data from another object.
	 * @param book The object from which the data is copied.
	 */
	public void copy(Book book) {
		this.name = book.name;
		this.authors = book.authors;
	}

	/* @see java.lang.Comparable#compareTo(java.lang.Object) */
	public int compareTo(Book object) {
		return name.compareTo(object.name);
	}
}
