/** Bookshelf - Loan.java - 15/03/2007 */
package net.java.dev.esjug.bookshelf.domain;

import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQuery;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import net.java.dev.esjug.util.persistence.PersistentObjectSupport;

/**
 * TODO: documentation.
 *
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
@Entity
@NamedQuery(name="loan.retrieveOpenLoans", query="from Loan as loan where loan.returnDate is null")
public class Loan extends PersistentObjectSupport<Long, Long> implements Comparable<Loan> {
	/** Serialization id. */
	private static final long serialVersionUID = 5840650648465751399L;

	/** Member that borrowed the book. */
	@ManyToOne
	private Member borrower;
	
	/** Book that has been borrowed. */
	@ManyToOne
	private Book book;
	
	/** Date the book was borrowed. */
	@Temporal(TemporalType.TIMESTAMP)
	private Date loanDate = new Date(System.currentTimeMillis());
	
	/** Date the book was returned. */
	@Temporal(TemporalType.TIMESTAMP)
	private Date returnDate;
	
	/** Empty constructor. */
	public Loan() { }
	
	/**
	 * Constructor.
	 * @param borrower Member that is borrowing the book.
	 * @param book Book that is being borrowed.
	 */
	public Loan(Member borrower, Book book) {
		this.borrower = borrower;
		this.book = book;
		book.addLoan(this);
	}

	/**
	 * Getter for book.
	 * @return The book
	 */
	public Book getBook() {
		return book;
	}

	/**
	 * Setter for book.
	 * @param book The book to set.
	 */
	@SuppressWarnings("unused")
	private void setBook(Book book) {
		this.book = book;
	}

	/**
	 * Getter for borrower.
	 * @return The borrower
	 */
	public Member getBorrower() {
		return borrower;
	}

	/**
	 * Setter for borrower.
	 * @param borrower The borrower to set.
	 */
	@SuppressWarnings("unused")
	private void setBorrower(Member borrower) {
		this.borrower = borrower;
	}

	/**
	 * Getter for loanDate.
	 * @return The loanDate
	 */
	public Date getLoanDate() {
		return loanDate;
	}

	/**
	 * Setter for loanDate.
	 * @param loanDate The loanDate to set.
	 */
	@SuppressWarnings("unused")
	private void setLoanDate(Date loanDate) {
		this.loanDate = loanDate;
	}

	/**
	 * Getter for returnDate.
	 * @return The returnDate
	 */
	public Date getReturnDate() {
		return returnDate;
	}

	/**
	 * Setter for returnDate.
	 * @param returnDate The returnDate to set.
	 */
	@SuppressWarnings("unused")
	private void setReturnDate(Date returnDate) {
		this.returnDate = returnDate;
	}

	/**
	 * Indicates if the loan is closed.
	 * 
	 * @return <code>true</code>, if the loan is closed (the book has been returned), <code>false</code> otherwise.
	 */
	public boolean isClosed() {
		return returnDate != null;
	}

	/* @see java.lang.Comparable#compareTo(java.lang.Object) */
	public int compareTo(Loan object) {
		return loanDate.compareTo(object.loanDate);
	}
	
	/**
	 * Mark the book as returned and finishes the loan.
	 */
	public void returnBook() {
		returnDate = new Date(System.currentTimeMillis());
	}
}
